"""
Following scripts are usable as Pythonista action items (wrench menu icon). In other
words, you can use them even without external keyboard.


Action quickly
==============

.. automodule:: blackmamba.script.action_quickly

Analyze
=======

.. automodule:: blackmamba.script.analyze

Clear annotations
=================

.. automodule:: blackmamba.script.clear_annotations

Close all tabs except current one
=================================

.. automodule:: blackmamba.script.close_all_tabs_except_current_one


Drag & Drop
===========

.. automodule:: blackmamba.script.drag_and_drop


Find usages
===========

.. automodule:: blackmamba.script.find_usages


Jump to definition
==================

.. automodule:: blackmamba.script.jump_to_definition


Jump to line
============

.. automodule:: blackmamba.script.jump_to_line


New file
========

.. automodule:: blackmamba.script.new_file


New tab
=======

.. automodule:: blackmamba.script.new_tab


Open quickly
============

.. automodule:: blackmamba.script.open_quickly


Outline quickly
===============

.. automodule:: blackmamba.script.outline_quickly


Run quickly
===========

.. automodule:: blackmamba.script.run_quickly


Run unit tests
==============
.. automodule:: blackmamba.script.run_unit_tests


Search Dash
===========

.. automodule:: blackmamba.script.search_dash


Show documentation
==================

.. automodule:: blackmamba.script.show_documentation


Toggle comments
===============

.. automodule:: blackmamba.script.toggle_comments
"""
